local Class = require('lib.hump.class')
local Anim8 = require('lib.anim8')

Player = Class{}

function Player:init(world, x, y, w, h)
   self.world = world
   self.posX = x
   self.posY = y
   self.w = w
   self.h = h

   self.animeSpeed = 4
   self.walkSpeed  = 400
   --self.sprintSpeed = 500

   self.world:addCollisionClass('Player')
   self.world:addCollisionClass('PlayerIgnoresPlatform', {ignores = {'Platform'}})

   self.collider = world:newRectangleCollider(self.posX, self.posY, self.w, self.h, {collision_class = 'Player'})


   self.imageIdle = love.graphics.newImage('resource/spritesheet/hero-idle.png')
   self.imageRun  = love.graphics.newImage('resource/spritesheet/hero-run.png')
   self.imageJump = love.graphics.newImage('resource/spritesheet/hero-jump.png')

   local g = Anim8.newGrid(32, 32, self.imageIdle:getWidth(), self.imageIdle:getHeight())
   self.animeIdle = Anim8.newAnimation(g('1-1', 1), 1)

   g = Anim8.newGrid(39, 32, self.imageRun:getWidth(), self.imageRun:getHeight())
   self.animeRun = Anim8.newAnimation(g('1-5', 1), 1)

   g = Anim8.newGrid(38, 32, self.imageJump:getWidth(), self.imageJump:getHeight())
   self.animeJump = Anim8.newAnimation(g('1-1', 1), 1)

   self.flip = true
end

function Player:update(dt)
   
   local speedX, speedY = self.collider.body:getLinearVelocity()

   self.posX, self.posY = self.collider.body:getPosition()

   -- Deceleration multiplyer to self.animeSpeed
   if not love.keyboard.isDown('a', 'd') then
      if self.animeSpeed > 6 then
         self.animeSpeed = self.animeSpeed - 0.25
      else
         self.animeSpeed = 4
      end
   end

   -- Move left
   if love.keyboard.isDown('a') then
      self:walkLeft(speedX, speedY)
   end

   -- Move right
   if love.keyboard.isDown('d') then
      self:walkRight(speedX, speedY)
   end

   -- Animation updates
   self.animeRun:update(dt*self.animeSpeed)
   self.animeJump:update(dt)
   self.animeIdle:update(dt)
end

function Player:draw()
   local speedX, speedY = self.collider.body:getLinearVelocity()
   
   -- Checks whether character is in the air or not
   if speedY < -1 or speedY > 1 then 
      self.animeJump:draw(self.imageJump, self.posX-(self.w/2), self.posY-(self.h/2))
   else
      -- Checks if character is moving or not
      if speedX < -1 or speedX > 1 then
         self.animeRun:draw(self.imageRun, self.posX-(self.w/2), self.posY-(self.h/2))
      else 
         self.animeIdle:draw(self.imageIdle, self.posX-(self.w/2), self.posY-(self.h/2))
         self.animeSpeed = 4
      end
   end
   love.graphics.setColor(255, 255, 255)
end

function Player:jump()
   -- Jumping impulse
   self.collider.body:applyLinearImpulse(0, -1200)
   --self.animeSpeed = 4
end

function Player:walkLeft(sx, sy)
   -- Checks if player is on the ground
   if sy < 5 and sy > -5 then
      -- checks if our current speed is lower than the max walk speed
      if math.abs(sx) < self.walkSpeed then
         self.collider.body:applyForce(-1200, sy)
         self.animeSpeed = self.animeSpeed + 0.15
      else
         self.collider.body:setLinearVelocity(-self.walkSpeed, 0)
         self.animeSpeed = 20
      end
   end
end

-- Same as Player:walkLeft
function Player:walkRight(sx, sy)
   if sy < 5 and sy > -5 then
      if math.abs(sx) < self.walkSpeed then
         self.collider.body:applyForce(1200, sy)
         self.animeSpeed = self.animeSpeed + 0.15
      else
         self.collider.body:setLinearVelocity(self.walkSpeed, 0)
         self.animeSpeed = 20
      end
   end
end

function Player:keypressed(key)
   -- Applys the jump method
   if key == 'space' then
      self:jump()
   end

   -- Checks what direction the character is facing
   if (key == 'a' and self.flip == true) or (key == 'd' and self.flip == false) then
      self.animeRun:flipH()
      self.animeJump:flipH()
      self.animeIdle:flipH()
      self.flip = not self.flip
   end
end

function Player:getCenter()
   return self.posX + self.w / 2,
          self.posY + self.h / 2
end

return Player